"""Tkinter based GUI for pythondoc.

Requires pmw!
"""

__author__ = "Daniel Larsson, Daniel.Larsson@telia.com"

import Tkinter

DOWNLOAD_INFO = """
pythondoc GUI needs the following components to work:

tkFileDialog - Fetch it from Fredrik Lundh's home page at
               http://starship.skyport.net/crew/fredrik

Pmw -          Peter Munnings' and Greg McFarlane's Python megawidgets.
               http://www.dscpl.com.au/pmw
"""

try:
    import Pmw
    import tkFileDialog
except ImportError:
    print DOWNLOAD_INFO
    import sys
    print "%s: %s" % (sys.exc_type, sys.exc_value)
    sys.exit(-1)

# pythondoc modules
import pythondoc, version, options



COPYRIGHT = "Copyright 1998 %s" % __author__

CONTACT = """
Email: Daniel.Larsson@telia.com
WWW: http://starship.python.net/crew/danilo/pythondoc
"""

Pmw.aboutversion(version.__version__)
Pmw.aboutcopyright(COPYRIGHT)
Pmw.aboutcontact(CONTACT)

class MainWindow(Tkinter.Toplevel):

    def __init__(self, master, startup_display=None):
	Tkinter.Toplevel.__init__(self, master)
	self.withdraw()

	self.startup_display = startup_display

	self.__title = "%s, %s" % ("pythondoc", version.__version__)
	self.title(self.__title)
	
	self.__menubar()
	self.__bindings()
	self.__make_children()

	# The about dialog
	self.__about_dlg = Pmw.AboutDialog(master,
					   applicationname="pythondoc")
	self.__about_dlg.withdraw()
	self.deiconify()

    def __make_children(self):
	self.__notebook = Pmw.NoteBookR(self)
	self.__notebook.add("main", label="Main")
	page = self.__notebook.main
	self.__setup_formatter_page(page)
	self.__notebook.pack(expand=1, fill='both')
	import formatters.format_init
	for formatter in formatters.format_init.formatters.keys():
	    if formatter[0] != "_":
		self.__notebook.add(formatter, label=formatter)
	        page = getattr(self.__notebook, formatter)
	        self.__setup_formatter_page(page)

    def __setup_formatter_page(self, page):
        frame = page.interior()
        boolframe = Tkinter.Frame(frame)
        boolframe.pack(expand=1, fill='x')
        dirframe = Tkinter.Frame(frame)
        dirframe.pack(expand=1, fill='x')
        stringframe = Tkinter.Frame(frame)
        stringframe.pack(expand=1, fill='x')
        enumframe = Tkinter.Frame(frame)
        enumframe.pack(expand=1, fill='x')
        intframe = Tkinter.Frame(frame)
        intframe.pack(expand=1, fill='x')
        
        if page.pagename != 'main':
	    check = Tkinter.Checkbutton(boolframe, text="Generate %s" % page.pagename)
	    check.pack(anchor='w')
	opts = options.options().options.get(page.pagename, {})
	print opts
	for key, (type, description, default, values) in opts.items():
	    print page.pagename, key
	    if type == options.Options.BOOL:
	        w = Tkinter.Checkbutton(boolframe, text=description)
	        if default:
	            w.select()
	    elif type in (options.Options.STRING, options.Options.URL):
	        w = Pmw.EntryField(stringframe, labelpos='w',
	                           label_text=description, value=default)
	    elif type == options.Options.INT:
	        w = Pmw.EntryField(intframe,
	        		   labelpos='w',
	                           label_text=description,
	                           value=default,
	                           validate=Pmw.integervalidator)
	    elif type in (options.Options.FILE, options.Options.DIR):
	        w = Tkinter.Frame(dirframe)
	        entry = Pmw.EntryField(w, labelpos='w', value=default, label_text=description)
	        entry.pack(side='left')
	        button = Tkinter.Button(w, text="...")
	        button.pack(side='left')
	    elif type == options.Options.ENUM:
	        w = Pmw.OptionMenu(enumframe, labelpos='w',
	                           label_text=description,
	                           items=values,
	                           initialitem=list(values).index(default))
	    else:
	        w = Tkinter.Label(frame, text=description)

	    w.pack(anchor="w")


    def __menubar(self):
	menub = Tkinter.Menu(self, tearoff=Tkinter.NO)
	menu = Tkinter.Menu(menub, tearoff=Tkinter.NO)
	menub.add_cascade(label='File', underline=0, menu=menu)

	menu.add_command(label='Open...',
			 command=self.__open_cmd,
			 accelerator='Ctrl+O')
	menu.add_separator()
	menu.add_command(label='Exit',
			 command=self.__exit_cmd,
			 accel='Ctrl+X')

	menu = Tkinter.Menu(menub, tearoff=Tkinter.NO)
	menub.add_cascade(label='Help', underline=0, menu=menu)

	menu.add_command(label='About...', command=self.__about)

	self.configure(menu=menub)

    def __bindings(self):
	self.bind('<Control-o>',self.__open_cmd)
	self.bind_all('<Control-x>',self.__exit_cmd)

    def __about(self):
	self.__about_dlg.activate()

    def __open_cmd(self, event=None):
	from tkFileDialog import askopenfilename
	filename = askopenfilename(defaultextension='.py',
				   filetypes=(('Python file', '.py'),),
				   parent = self,
				   title = "Open File")

	if not filename: return


    def __exit_cmd(self, event=None):
	if self.changed:
	    if not self.__asksave():
		return
	self.tk.quit()


def run(formatters=None):
    root = Tkinter.Tk(className='pythondoc')
    root.withdraw()

    m = MainWindow(root)
    root.mainloop()
